#
import grpc
from google.protobuf import empty_pb2 as empty_request
from LedgerCompliance.schema import lc_pb2_grpc as lc
from LedgerCompliance.schema import schema_pb2_grpc as immudb
from LedgerCompliance.schema.schema_pb2_grpc import schema__pb2 as schema_pb2
from LedgerCompliance.schema.lc_pb2_grpc import lc__pb2 as lc_pb2

from LedgerCompliance import header_manipulator_client_interceptor as interceptor
from . import types, proofs, utils, constants
import time

class Client:
	def __init__(self, apikey: str, host: str, port: int, secure:bool=True):
		self.apikey=apikey
		self.host=host
		self.port=port
		if secure:
			self.credentials=grpc.ssl_channel_credentials()
			self.channel = grpc.secure_channel(
				"{}:{}".format(self.host,self.port),self.credentials)
		else:
			self.channel = grpc.insecure_channel(
				"{}:{}".format(self.host,self.port))
		self.__stub = lc.LcServiceStub(self.channel)
		self.__rs = None
		
	def set_credentials(self, root_certificates=None, private_key=None, certificate_chain=None):
		self.credentials=grpc.ssl_channel_credentials(
			root_certificates, private_key, certificate_chain)
		self.channel = grpc.secure_channel(
			"{}:{}".format(self.host,self.port),self.credentials)
		self.__stub = lc.LcServiceStub(self.channel)
		self.__rs = None

	def set_interceptor(self, apikey):
		""" Set up an interceptor so all grpc calls will have the apikey
		added on the header, in order to authenticate.
		"""
		self.header_interceptor = \
		interceptor.header_adder_interceptor(
			'lc-api-key', apikey
		)
		try:
			self.intercept_channel = grpc.intercept_channel(
				self.channel, self.header_interceptor)
		except ValueError as e:
			raise Exception("Attempted to connect on termninated client, "
				"channel has been shutdown") from e
		return lc.LcServiceStub(self.intercept_channel)
	
	def connect(self):
		self.__stub=self.set_interceptor(self.apikey)
		return self.currentState()
	
	def currentState(self):
		resp=self.__stub.CurrentState(empty_request.Empty())
		self.__rs=types.LCState(
			db=resp.db,
			txid=resp.txId,
			txhash=resp.txHash
		)
		return self.__rs
		
	def set(self, key: bytes, value: bytes):
		request=schema_pb2.SetRequest(
			KVs=[schema_pb2.KeyValue(key=key, value=value)]
		)
		ret = self.__stub.Set(request)
		return types.LCIndex(txid=ret.id)
	
	def get(self, key:bytes) -> types.LCIndex:
		request = schema_pb2.Key(key=key)
		ret=self.__stub.Get(request)
		return types.LCItem(key=ret.key, value=ret.value, txid=ret.tx)
	
	def getValue(self, key:bytes) -> bytes:
		request = schema_pb2.Key(key=key)
		ret=self.__stub.Get(request)
		return ret.value

	def verifiedSet(self, key: bytes, value: bytes):
		# print(base64.b64encode(state.SerializeToString()))
		kv = schema_pb2.KeyValue(key=key, value=value)
		rawRequest = schema_pb2.VerifiableSetRequest(
			setRequest = schema_pb2.SetRequest(KVs=[kv]),
			proveSinceTx= self.__rs.txid,
		)
		verifiableTx = self.__stub.VerifiableSet(rawRequest)
		# print(base64.b64encode(verifiableTx.SerializeToString()))
		tx=proofs.TxFrom(verifiableTx.tx)
		inclusionProof=tx.Proof(constants.SET_KEY_PREFIX+key)
		ekv=proofs.EncodeKV(key, value)
		verifies=proofs.VerifyInclusion(inclusionProof, ekv.Digest(), tx.eh())
		if not verifies:
			raise types.VerificationException
		if tx.eh() != proofs.DigestFrom(verifiableTx.dualProof.targetTxMetadata.eH):
			raise types.VerificationException
		if self.__rs.txid == 0:
			sourceID = tx.ID
			sourceAlh = tx.Alh
		else:
			sourceID = self.__rs.txid
			sourceAlh = proofs.DigestFrom(self.__rs.txhash)
		targetID = tx.ID
		targetAlh = tx.Alh

		verifies = proofs.VerifyDualProof( proofs.DualProofFrom(verifiableTx.dualProof), sourceID, targetID, sourceAlh, targetAlh, )
		if not verifies:
			raise types.VerificationException
		self.__rs.txid=targetID
		self.__rs.txhash=targetAlh
		return types.SafeSetResponse(
			txid=targetID,
			verified=verifies,
		)
	
	def verifiedGet(self, requestkey: bytes, atTx:int=None):
		if atTx==None:
			req = schema_pb2.VerifiableGetRequest(
			keyRequest= schema_pb2.KeyRequest(key=requestkey),
			proveSinceTx= self.__rs.txid
			)
		else:
			req = schema_pb2.VerifiableGetRequest(
			keyRequest= schema_pb2.KeyRequest(key=requestkey, atTx=atTx),
			proveSinceTx= self.__rs.txid
			)
		ventry=self.__stub.VerifiableGet(req)
		inclusionProof = proofs.InclusionProofFrom(ventry.inclusionProof)
		dualProof = proofs.DualProofFrom(ventry.verifiableTx.dualProof)
		
		if ventry.entry.referencedBy==None or ventry.entry.referencedBy.key==b'':
			vTx=ventry.entry.tx
			kv=proofs.EncodeKV(requestkey, ventry.entry.value)
		else:
			vTx = ventry.entry.referencedBy.tx
			kv=proofs.EncodeReference(ventry.entry.referencedBy.key, ventry.entry.key, ventry.entry.referencedBy.atTx) 
			
		if self.__rs.txid <= vTx:
			eh=proofs.DigestFrom(ventry.verifiableTx.dualProof.targetTxMetadata.eH)
			sourceid=self.__rs.txid
			sourcealh=proofs.DigestFrom(self.__rs.txhash)
			targetid=vTx
			targetalh=dualProof.targetTxMetadata.alh()
		else:
			eh=proofs.DigestFrom(ventry.verifiableTx.dualProof.sourceTxMetadata.eH)
			sourceid=vTx
			sourcealh=dualProof.sourceTxMetadata.alh()
			targetid=self.__rs.txid
			targetalh=proofs.DigestFrom(self.__rs.txhash)
			
		verifies = proofs.VerifyInclusion(inclusionProof,kv.Digest(),eh)
		if not verifies:
			raise VerificationException
		verifies=proofs.VerifyDualProof( dualProof, sourceid, targetid, sourcealh, targetalh)
		if not verifies:
			raise VerificationException
		self.__rs.txid=targetid
		self.__rs.txhash=targetalh
		if ventry.entry.referencedBy!=None and ventry.entry.referencedBy.key!=b'':
			refkey=ventry.entry.referencedBy.key
		else:
			refkey=None
		return types.SafeGetResponse(
			txid=vTx,
			key=ventry.entry.key,
			value=ventry.entry.value,
			timestamp=ventry.verifiableTx.tx.metadata.ts,
			verified=verifies,
			refkey=refkey
			)
	def health(self):
		resp=self.__stub.Health(empty_request.Empty())
		return types.HealthInfo(status=resp.status, version=resp.version)
	
	def setBatch(self, kv: dict):
		ops=[]
		for (key,value) in kv.items():
			xreq=schema_pb2.Op(
					kv=schema_pb2.KeyValue(key=key, value=value)
					)
			ops.append(xreq)
		request=schema_pb2.ExecAllRequest(
			Operations=ops
		)		
		ret=self.__stub.ExecAll(request)
		return types.LCIndex(txid=ret.id)
	
	def getBatch(self, keys: list):
		request = schema_pb2.KeyListRequest(keys=keys)
		ret=self.__stub.GetAll(request)
		return [types.LCItem(key=t.key, value=t.value, txid=t.tx) for t in ret.entries]
	
	def getValueBatch(self, keys: list):
		request = schema_pb2.KeyListRequest(keys=keys)
		ret=self.__stub.GetAll(request)
		return {t.key:t.value for t in ret.entries}
	
	def scan(self, seekKey:bytes, prefix:bytes, desc:bool=False, limit:int=10, sinceTx:int=None, noWait:bool=False):
		request = schema_pb2.ScanRequest(
			seekKey = seekKey ,
			prefix = prefix, 
			desc = desc, 
			limit = limit, 
			sinceTx = sinceTx,
			noWait = noWait
			)
		ret=self.__stub.Scan(request)
		return [types.LCItem(key=t.key, value=t.value, txid=t.tx) for t in ret.entries]
	
	def history(self, key: bytes, offset:int=0, limit:int=10, desc:bool=False, sinceTx:int=0):
		request = schema_pb2.HistoryRequest(
			key=key, offset=offset, limit=limit, desc=desc, sinceTx=sinceTx)
		ret=self.__stub.History(request)
		return [types.LCItem(key=t.key, value=t.value, txid=t.tx) for t in ret.entries]
	
	def zAdd(self, zset:bytes, score:float, key:bytes, atTx:int=0):
		request = schema_pb2.ZAddRequest(set=zset, score=score, key=key, atTx=atTx,
				   boundRef=atTx>0)
		ret= self.__stub.ZAdd(request)
		return types.LCIndex(txid=ret.id)
	
	def verifiedZAdd(self, zset:bytes, score:float, key:bytes, atTx:int=0):
		request=schema_pb2.VerifiableZAddRequest(
			zAddRequest=schema_pb2.ZAddRequest(
			set=      zset,
			score=    score,
			key=      key,
			atTx=     atTx,
			),
			proveSinceTx=self.__rs.txid
			)
		vtx = self.__stub.VerifiableZAdd(request)
		if vtx.tx.metadata.nentries!=1:
			raise VerificationException
		tx = proofs.TxFrom(vtx.tx)
		ekv = proofs.EncodeZAdd(zset, score, key, atTx)
		inclusionProof=tx.Proof(ekv.key)
		verifies = proofs.VerifyInclusion(inclusionProof, ekv.Digest(), tx.eh())
		if not verifies:
			raise VerificationException
		if tx.eh() != proofs.DigestFrom(vtx.dualProof.targetTxMetadata.eH):
			raise VerificationException
		if self.__rs.txid == 0:
			sourceID = tx.ID
			sourceAlh = tx.Alh
		else:
			sourceID = self.__rs.txid
			sourceAlh = proofs.DigestFrom(self.__rs.txhash)
		targetID = tx.ID
		targetAlh = tx.Alh
		verifies = proofs.VerifyDualProof(
			proofs.DualProofFrom(vtx.dualProof),
			sourceID,
			targetID,
			sourceAlh,
			targetAlh,
		)
		if not verifies:
			raise VerificationException

		self.__rs.txid=targetID
		self.__rs.txhash=targetAlh
		return types.SafeSetResponse(
			txid=targetID,
			verified=verifies,
		)
	
	def zScan(self, zset:bytes, seekKey:bytes, seekScore:float, 
			inclusiveSeek:bool=False, seekAtTx:int=0, limit:int=10, desc:bool=False,
			minScore:float=None, maxScore:float=None, sinceTx:int=0, noWait:bool=False):
		
		request=schema_pb2.ZScanRequest(set=zset, seekKey=seekKey, seekScore=seekScore,
				  seekAtTx=seekAtTx, inclusiveSeek=inclusiveSeek, limit=limit, desc=desc,
				  minScore=schema_pb2.Score(score=minScore),
				  maxScore=schema_pb2.Score(score=maxScore), 
				  sinceTx=sinceTx, noWait=noWait)
				  
		ret=self.__stub.ZScan(request)
		print(ret)
		return [types.ZItem(key=t.entry.key, value=t.entry.value, txid=t.entry.tx, score=t.score) for t in ret.entries]
	
	def reportTamper(self, index:int, key:bytes, signature:bytes=None, publickey:bytes=None):
		report=lc_pb2.TamperReport(index=index, key=key, root=self.__rs.root)
		signature=schema_pb2.Signature(signature=signature, publicKey=publickey)
		request=lc_pb2.ReportOptions(payload=report, signature=signature)
		self.__stub.ReportTamper(request)
	




from dataclasses import dataclass

@dataclass
class LCState:
	db: str
	txid: int
	txhash: bytes
	
@dataclass
class LCIndex:
	txid: int
	
@dataclass
class LCItem:
	txid: int
	key: bytes
	value: bytes

@dataclass
class ZItem:
	txid: int
	key: bytes
	value: bytes
	score: float

    
@dataclass
class SafeSetResponse:
	txid: int
	verified: bool
    
@dataclass
class SafeGetResponse:
    txid: int
    key: bytes
    value: bytes
    timestamp: int
    verified: bool
    refkey: bytes
    
@dataclass    
class HealthInfo:
	status: bool
	version: str


# exceptions
class VerificationException(Exception):
	pass

class ErrMaxWidthExceeded(Exception):
	pass

class ErrIllegalArguments(Exception):
	pass




# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2
from . import lc_pb2 as lc__pb2
from . import schema_pb2 as schema__pb2


class LcServiceStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.Set = channel.unary_unary(
                '/lc.schema.LcService/Set',
                request_serializer=schema__pb2.SetRequest.SerializeToString,
                response_deserializer=schema__pb2.TxMetadata.FromString,
                )
        self.Get = channel.unary_unary(
                '/lc.schema.LcService/Get',
                request_serializer=schema__pb2.KeyRequest.SerializeToString,
                response_deserializer=schema__pb2.Entry.FromString,
                )
        self.VerifiableSet = channel.unary_unary(
                '/lc.schema.LcService/VerifiableSet',
                request_serializer=schema__pb2.VerifiableSetRequest.SerializeToString,
                response_deserializer=schema__pb2.VerifiableTx.FromString,
                )
        self.VerifiableGet = channel.unary_unary(
                '/lc.schema.LcService/VerifiableGet',
                request_serializer=schema__pb2.VerifiableGetRequest.SerializeToString,
                response_deserializer=schema__pb2.VerifiableEntry.FromString,
                )
        self.GetAll = channel.unary_unary(
                '/lc.schema.LcService/GetAll',
                request_serializer=schema__pb2.KeyListRequest.SerializeToString,
                response_deserializer=schema__pb2.Entries.FromString,
                )
        self.ExecAll = channel.unary_unary(
                '/lc.schema.LcService/ExecAll',
                request_serializer=schema__pb2.ExecAllRequest.SerializeToString,
                response_deserializer=schema__pb2.TxMetadata.FromString,
                )
        self.Scan = channel.unary_unary(
                '/lc.schema.LcService/Scan',
                request_serializer=schema__pb2.ScanRequest.SerializeToString,
                response_deserializer=schema__pb2.Entries.FromString,
                )
        self.History = channel.unary_unary(
                '/lc.schema.LcService/History',
                request_serializer=schema__pb2.HistoryRequest.SerializeToString,
                response_deserializer=schema__pb2.Entries.FromString,
                )
        self.ZAdd = channel.unary_unary(
                '/lc.schema.LcService/ZAdd',
                request_serializer=schema__pb2.ZAddRequest.SerializeToString,
                response_deserializer=schema__pb2.TxMetadata.FromString,
                )
        self.VerifiableZAdd = channel.unary_unary(
                '/lc.schema.LcService/VerifiableZAdd',
                request_serializer=schema__pb2.VerifiableZAddRequest.SerializeToString,
                response_deserializer=schema__pb2.VerifiableTx.FromString,
                )
        self.ZScan = channel.unary_unary(
                '/lc.schema.LcService/ZScan',
                request_serializer=schema__pb2.ZScanRequest.SerializeToString,
                response_deserializer=schema__pb2.ZEntries.FromString,
                )
        self.CurrentState = channel.unary_unary(
                '/lc.schema.LcService/CurrentState',
                request_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
                response_deserializer=schema__pb2.ImmutableState.FromString,
                )
        self.Health = channel.unary_unary(
                '/lc.schema.LcService/Health',
                request_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
                response_deserializer=schema__pb2.HealthResponse.FromString,
                )
        self.ReportTamper = channel.unary_unary(
                '/lc.schema.LcService/ReportTamper',
                request_serializer=lc__pb2.ReportOptions.SerializeToString,
                response_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
                )
        self.SendData = channel.stream_stream(
                '/lc.schema.LcService/SendData',
                request_serializer=lc__pb2.Data.SerializeToString,
                response_deserializer=lc__pb2.Response.FromString,
                )
        self.VerifiableGetExt = channel.unary_unary(
                '/lc.schema.LcService/VerifiableGetExt',
                request_serializer=schema__pb2.VerifiableGetRequest.SerializeToString,
                response_deserializer=lc__pb2.VerifiableItemExt.FromString,
                )
        self.ZScanExt = channel.unary_unary(
                '/lc.schema.LcService/ZScanExt',
                request_serializer=schema__pb2.ZScanRequest.SerializeToString,
                response_deserializer=lc__pb2.ZItemExtList.FromString,
                )
        self.HistoryExt = channel.unary_unary(
                '/lc.schema.LcService/HistoryExt',
                request_serializer=schema__pb2.HistoryRequest.SerializeToString,
                response_deserializer=lc__pb2.ItemExtList.FromString,
                )


class LcServiceServicer(object):
    """Missing associated documentation comment in .proto file."""

    def Set(self, request, context):
        """immudb primitives
        setters and getters
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Get(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def VerifiableSet(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def VerifiableGet(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetAll(self, request, context):
        """batch
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ExecAll(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Scan(self, request, context):
        """scanners
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def History(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ZAdd(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def VerifiableZAdd(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ZScan(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CurrentState(self, request, context):
        """mixed
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Health(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ReportTamper(self, request, context):
        """ledger compliance extensions
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SendData(self, request_iterator, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def VerifiableGetExt(self, request, context):
        """ledger compliance extensions - items extended with additional properties managed by LC backend (date)
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ZScanExt(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def HistoryExt(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_LcServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'Set': grpc.unary_unary_rpc_method_handler(
                    servicer.Set,
                    request_deserializer=schema__pb2.SetRequest.FromString,
                    response_serializer=schema__pb2.TxMetadata.SerializeToString,
            ),
            'Get': grpc.unary_unary_rpc_method_handler(
                    servicer.Get,
                    request_deserializer=schema__pb2.KeyRequest.FromString,
                    response_serializer=schema__pb2.Entry.SerializeToString,
            ),
            'VerifiableSet': grpc.unary_unary_rpc_method_handler(
                    servicer.VerifiableSet,
                    request_deserializer=schema__pb2.VerifiableSetRequest.FromString,
                    response_serializer=schema__pb2.VerifiableTx.SerializeToString,
            ),
            'VerifiableGet': grpc.unary_unary_rpc_method_handler(
                    servicer.VerifiableGet,
                    request_deserializer=schema__pb2.VerifiableGetRequest.FromString,
                    response_serializer=schema__pb2.VerifiableEntry.SerializeToString,
            ),
            'GetAll': grpc.unary_unary_rpc_method_handler(
                    servicer.GetAll,
                    request_deserializer=schema__pb2.KeyListRequest.FromString,
                    response_serializer=schema__pb2.Entries.SerializeToString,
            ),
            'ExecAll': grpc.unary_unary_rpc_method_handler(
                    servicer.ExecAll,
                    request_deserializer=schema__pb2.ExecAllRequest.FromString,
                    response_serializer=schema__pb2.TxMetadata.SerializeToString,
            ),
            'Scan': grpc.unary_unary_rpc_method_handler(
                    servicer.Scan,
                    request_deserializer=schema__pb2.ScanRequest.FromString,
                    response_serializer=schema__pb2.Entries.SerializeToString,
            ),
            'History': grpc.unary_unary_rpc_method_handler(
                    servicer.History,
                    request_deserializer=schema__pb2.HistoryRequest.FromString,
                    response_serializer=schema__pb2.Entries.SerializeToString,
            ),
            'ZAdd': grpc.unary_unary_rpc_method_handler(
                    servicer.ZAdd,
                    request_deserializer=schema__pb2.ZAddRequest.FromString,
                    response_serializer=schema__pb2.TxMetadata.SerializeToString,
            ),
            'VerifiableZAdd': grpc.unary_unary_rpc_method_handler(
                    servicer.VerifiableZAdd,
                    request_deserializer=schema__pb2.VerifiableZAddRequest.FromString,
                    response_serializer=schema__pb2.VerifiableTx.SerializeToString,
            ),
            'ZScan': grpc.unary_unary_rpc_method_handler(
                    servicer.ZScan,
                    request_deserializer=schema__pb2.ZScanRequest.FromString,
                    response_serializer=schema__pb2.ZEntries.SerializeToString,
            ),
            'CurrentState': grpc.unary_unary_rpc_method_handler(
                    servicer.CurrentState,
                    request_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
                    response_serializer=schema__pb2.ImmutableState.SerializeToString,
            ),
            'Health': grpc.unary_unary_rpc_method_handler(
                    servicer.Health,
                    request_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
                    response_serializer=schema__pb2.HealthResponse.SerializeToString,
            ),
            'ReportTamper': grpc.unary_unary_rpc_method_handler(
                    servicer.ReportTamper,
                    request_deserializer=lc__pb2.ReportOptions.FromString,
                    response_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            ),
            'SendData': grpc.stream_stream_rpc_method_handler(
                    servicer.SendData,
                    request_deserializer=lc__pb2.Data.FromString,
                    response_serializer=lc__pb2.Response.SerializeToString,
            ),
            'VerifiableGetExt': grpc.unary_unary_rpc_method_handler(
                    servicer.VerifiableGetExt,
                    request_deserializer=schema__pb2.VerifiableGetRequest.FromString,
                    response_serializer=lc__pb2.VerifiableItemExt.SerializeToString,
            ),
            'ZScanExt': grpc.unary_unary_rpc_method_handler(
                    servicer.ZScanExt,
                    request_deserializer=schema__pb2.ZScanRequest.FromString,
                    response_serializer=lc__pb2.ZItemExtList.SerializeToString,
            ),
            'HistoryExt': grpc.unary_unary_rpc_method_handler(
                    servicer.HistoryExt,
                    request_deserializer=schema__pb2.HistoryRequest.FromString,
                    response_serializer=lc__pb2.ItemExtList.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'lc.schema.LcService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class LcService(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def Set(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/lc.schema.LcService/Set',
            schema__pb2.SetRequest.SerializeToString,
            schema__pb2.TxMetadata.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Get(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/lc.schema.LcService/Get',
            schema__pb2.KeyRequest.SerializeToString,
            schema__pb2.Entry.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def VerifiableSet(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/lc.schema.LcService/VerifiableSet',
            schema__pb2.VerifiableSetRequest.SerializeToString,
            schema__pb2.VerifiableTx.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def VerifiableGet(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/lc.schema.LcService/VerifiableGet',
            schema__pb2.VerifiableGetRequest.SerializeToString,
            schema__pb2.VerifiableEntry.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetAll(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/lc.schema.LcService/GetAll',
            schema__pb2.KeyListRequest.SerializeToString,
            schema__pb2.Entries.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ExecAll(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/lc.schema.LcService/ExecAll',
            schema__pb2.ExecAllRequest.SerializeToString,
            schema__pb2.TxMetadata.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Scan(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/lc.schema.LcService/Scan',
            schema__pb2.ScanRequest.SerializeToString,
            schema__pb2.Entries.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def History(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/lc.schema.LcService/History',
            schema__pb2.HistoryRequest.SerializeToString,
            schema__pb2.Entries.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ZAdd(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/lc.schema.LcService/ZAdd',
            schema__pb2.ZAddRequest.SerializeToString,
            schema__pb2.TxMetadata.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def VerifiableZAdd(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/lc.schema.LcService/VerifiableZAdd',
            schema__pb2.VerifiableZAddRequest.SerializeToString,
            schema__pb2.VerifiableTx.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ZScan(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/lc.schema.LcService/ZScan',
            schema__pb2.ZScanRequest.SerializeToString,
            schema__pb2.ZEntries.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CurrentState(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/lc.schema.LcService/CurrentState',
            google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            schema__pb2.ImmutableState.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Health(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/lc.schema.LcService/Health',
            google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            schema__pb2.HealthResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ReportTamper(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/lc.schema.LcService/ReportTamper',
            lc__pb2.ReportOptions.SerializeToString,
            google_dot_protobuf_dot_empty__pb2.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SendData(request_iterator,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.stream_stream(request_iterator, target, '/lc.schema.LcService/SendData',
            lc__pb2.Data.SerializeToString,
            lc__pb2.Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def VerifiableGetExt(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/lc.schema.LcService/VerifiableGetExt',
            schema__pb2.VerifiableGetRequest.SerializeToString,
            lc__pb2.VerifiableItemExt.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ZScanExt(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/lc.schema.LcService/ZScanExt',
            schema__pb2.ZScanRequest.SerializeToString,
            lc__pb2.ZItemExtList.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def HistoryExt(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/lc.schema.LcService/HistoryExt',
            schema__pb2.HistoryRequest.SerializeToString,
            lc__pb2.ItemExtList.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



#!/usr/bin/env python

from setuptools import setup

with open("../README.md", "rt") as fh:
    long_description = fh.read()

setup(name='ledger-compliance-py',
      version='2.1.5-rc1',
      description='Python SDK for codenotary Ledger Compliance',
      long_description=long_description,
      long_description_content_type="text/markdown",
      author='CodeNotary',
      url='https://github.com/vchain-us/ledger-compliance-py',
      packages=['LedgerCompliance', 'LedgerCompliance.schema',],
      install_requires=[
        'grpcio>=1.26.0',
        'dataclasses>=0.6',
        'protobuf>=3.13.0',
        'google-api>=0.1.12',
        'google-api-core>=1.22.0'
        ],
      classifiers=[
              'Intended Audience :: Developers',
              'Topic :: Software Development :: Build Tools',
              "License :: OSI Approved :: Apache Software License",
              "Operating System :: OS Independent",
              'Programming Language :: Python :: 3',
              'Programming Language :: Python :: 3.6',
              'Programming Language :: Python :: 3.7',
              'Programming Language :: Python :: 3.8',
              ],
      python_requires='>=3.6',
      )



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: lc.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import schema_pb2 as schema__pb2
from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='lc.proto',
  package='lc.schema',
  syntax='proto3',
  serialized_options=b'Z4github.com/vchain-us/ledger-compliance/server/schema',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x08lc.proto\x12\tlc.schema\x1a\x0cschema.proto\x1a\x1bgoogle/protobuf/empty.proto\x1a\x1fgoogle/protobuf/timestamp.proto\"8\n\x0cTamperReport\x12\r\n\x05index\x18\x01 \x01(\x04\x12\x0b\n\x03key\x18\x02 \x01(\x0c\x12\x0c\n\x04root\x18\x03 \x01(\x0c\"f\n\rReportOptions\x12(\n\x07payload\x18\x01 \x01(\x0b\x32\x17.lc.schema.TamperReport\x12+\n\tsignature\x18\x02 \x01(\x0b\x32\x18.immudb.schema.Signature\"6\n\tDataValue\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0c\n\x04type\x18\x02 \x01(\t\x12\r\n\x05value\x18\x03 \x01(\x0c\"f\n\x04\x44\x61ta\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05msgID\x18\x02 \x01(\x03\x12\x0e\n\x06\x64\x62Name\x18\x03 \x01(\t\x12\x11\n\ttableName\x18\x04 \x01(\t\x12\x11\n\toperation\x18\x05 \x01(\t\x12\x0c\n\x04\x64\x61ta\x18\x06 \x01(\x0c\"\x97\x01\n\x08Response\x12\x0e\n\x06status\x18\x01 \x01(\x05\x12\x0b\n\x03msg\x18\x02 \x01(\t\x12\r\n\x05msgID\x18\x03 \x01(\x03\x12\x31\n\x0cVerifiableTx\x18\x04 \x01(\x0b\x32\x1b.immudb.schema.VerifiableTx\x12,\n\x05state\x18\x05 \x01(\x0b\x32\x1d.immudb.schema.ImmutableState\"p\n\x11VerifiableItemExt\x12,\n\x04item\x18\x01 \x01(\x0b\x32\x1e.immudb.schema.VerifiableEntry\x12-\n\ttimestamp\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\"\\\n\x07ItemExt\x12\"\n\x04item\x18\x01 \x01(\x0b\x32\x14.immudb.schema.Entry\x12-\n\ttimestamp\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\"0\n\x0bItemExtList\x12!\n\x05items\x18\x01 \x03(\x0b\x32\x12.lc.schema.ItemExt\"^\n\x08ZItemExt\x12#\n\x04item\x18\x01 \x01(\x0b\x32\x15.immudb.schema.ZEntry\x12-\n\ttimestamp\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\"2\n\x0cZItemExtList\x12\"\n\x05items\x18\x01 \x03(\x0b\x32\x13.lc.schema.ZItemExt2\x82\n\n\tLcService\x12=\n\x03Set\x12\x19.immudb.schema.SetRequest\x1a\x19.immudb.schema.TxMetadata\"\x00\x12\x38\n\x03Get\x12\x19.immudb.schema.KeyRequest\x1a\x14.immudb.schema.Entry\"\x00\x12S\n\rVerifiableSet\x12#.immudb.schema.VerifiableSetRequest\x1a\x1b.immudb.schema.VerifiableTx\"\x00\x12V\n\rVerifiableGet\x12#.immudb.schema.VerifiableGetRequest\x1a\x1e.immudb.schema.VerifiableEntry\"\x00\x12\x41\n\x06GetAll\x12\x1d.immudb.schema.KeyListRequest\x1a\x16.immudb.schema.Entries\"\x00\x12\x45\n\x07\x45xecAll\x12\x1d.immudb.schema.ExecAllRequest\x1a\x19.immudb.schema.TxMetadata\"\x00\x12<\n\x04Scan\x12\x1a.immudb.schema.ScanRequest\x1a\x16.immudb.schema.Entries\"\x00\x12\x42\n\x07History\x12\x1d.immudb.schema.HistoryRequest\x1a\x16.immudb.schema.Entries\"\x00\x12?\n\x04ZAdd\x12\x1a.immudb.schema.ZAddRequest\x1a\x19.immudb.schema.TxMetadata\"\x00\x12U\n\x0eVerifiableZAdd\x12$.immudb.schema.VerifiableZAddRequest\x1a\x1b.immudb.schema.VerifiableTx\"\x00\x12?\n\x05ZScan\x12\x1b.immudb.schema.ZScanRequest\x1a\x17.immudb.schema.ZEntries\"\x00\x12G\n\x0c\x43urrentState\x12\x16.google.protobuf.Empty\x1a\x1d.immudb.schema.ImmutableState\"\x00\x12\x41\n\x06Health\x12\x16.google.protobuf.Empty\x1a\x1d.immudb.schema.HealthResponse\"\x00\x12\x42\n\x0cReportTamper\x12\x18.lc.schema.ReportOptions\x1a\x16.google.protobuf.Empty\"\x00\x12\x36\n\x08SendData\x12\x0f.lc.schema.Data\x1a\x13.lc.schema.Response\"\x00(\x01\x30\x01\x12W\n\x10VerifiableGetExt\x12#.immudb.schema.VerifiableGetRequest\x1a\x1c.lc.schema.VerifiableItemExt\"\x00\x12\x42\n\x08ZScanExt\x12\x1b.immudb.schema.ZScanRequest\x1a\x17.lc.schema.ZItemExtList\"\x00\x12\x45\n\nHistoryExt\x12\x1d.immudb.schema.HistoryRequest\x1a\x16.lc.schema.ItemExtList\"\x00\x42\x36Z4github.com/vchain-us/ledger-compliance/server/schemab\x06proto3'
  ,
  dependencies=[schema__pb2.DESCRIPTOR,google_dot_protobuf_dot_empty__pb2.DESCRIPTOR,google_dot_protobuf_dot_timestamp__pb2.DESCRIPTOR,])




_TAMPERREPORT = _descriptor.Descriptor(
  name='TamperReport',
  full_name='lc.schema.TamperReport',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='index', full_name='lc.schema.TamperReport.index', index=0,
      number=1, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='key', full_name='lc.schema.TamperReport.key', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='root', full_name='lc.schema.TamperReport.root', index=2,
      number=3, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=99,
  serialized_end=155,
)


_REPORTOPTIONS = _descriptor.Descriptor(
  name='ReportOptions',
  full_name='lc.schema.ReportOptions',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='payload', full_name='lc.schema.ReportOptions.payload', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='signature', full_name='lc.schema.ReportOptions.signature', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=157,
  serialized_end=259,
)


_DATAVALUE = _descriptor.Descriptor(
  name='DataValue',
  full_name='lc.schema.DataValue',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='lc.schema.DataValue.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='type', full_name='lc.schema.DataValue.type', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='lc.schema.DataValue.value', index=2,
      number=3, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=261,
  serialized_end=315,
)


_DATA = _descriptor.Descriptor(
  name='Data',
  full_name='lc.schema.Data',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='lc.schema.Data.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='msgID', full_name='lc.schema.Data.msgID', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='dbName', full_name='lc.schema.Data.dbName', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='tableName', full_name='lc.schema.Data.tableName', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='operation', full_name='lc.schema.Data.operation', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='data', full_name='lc.schema.Data.data', index=5,
      number=6, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=317,
  serialized_end=419,
)


_RESPONSE = _descriptor.Descriptor(
  name='Response',
  full_name='lc.schema.Response',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='status', full_name='lc.schema.Response.status', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='msg', full_name='lc.schema.Response.msg', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='msgID', full_name='lc.schema.Response.msgID', index=2,
      number=3, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='VerifiableTx', full_name='lc.schema.Response.VerifiableTx', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='state', full_name='lc.schema.Response.state', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=422,
  serialized_end=573,
)


_VERIFIABLEITEMEXT = _descriptor.Descriptor(
  name='VerifiableItemExt',
  full_name='lc.schema.VerifiableItemExt',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='item', full_name='lc.schema.VerifiableItemExt.item', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='timestamp', full_name='lc.schema.VerifiableItemExt.timestamp', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=575,
  serialized_end=687,
)


_ITEMEXT = _descriptor.Descriptor(
  name='ItemExt',
  full_name='lc.schema.ItemExt',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='item', full_name='lc.schema.ItemExt.item', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='timestamp', full_name='lc.schema.ItemExt.timestamp', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=689,
  serialized_end=781,
)


_ITEMEXTLIST = _descriptor.Descriptor(
  name='ItemExtList',
  full_name='lc.schema.ItemExtList',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='items', full_name='lc.schema.ItemExtList.items', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=783,
  serialized_end=831,
)


_ZITEMEXT = _descriptor.Descriptor(
  name='ZItemExt',
  full_name='lc.schema.ZItemExt',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='item', full_name='lc.schema.ZItemExt.item', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='timestamp', full_name='lc.schema.ZItemExt.timestamp', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=833,
  serialized_end=927,
)


_ZITEMEXTLIST = _descriptor.Descriptor(
  name='ZItemExtList',
  full_name='lc.schema.ZItemExtList',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='items', full_name='lc.schema.ZItemExtList.items', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=929,
  serialized_end=979,
)

_REPORTOPTIONS.fields_by_name['payload'].message_type = _TAMPERREPORT
_REPORTOPTIONS.fields_by_name['signature'].message_type = schema__pb2._SIGNATURE
_RESPONSE.fields_by_name['VerifiableTx'].message_type = schema__pb2._VERIFIABLETX
_RESPONSE.fields_by_name['state'].message_type = schema__pb2._IMMUTABLESTATE
_VERIFIABLEITEMEXT.fields_by_name['item'].message_type = schema__pb2._VERIFIABLEENTRY
_VERIFIABLEITEMEXT.fields_by_name['timestamp'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_ITEMEXT.fields_by_name['item'].message_type = schema__pb2._ENTRY
_ITEMEXT.fields_by_name['timestamp'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_ITEMEXTLIST.fields_by_name['items'].message_type = _ITEMEXT
_ZITEMEXT.fields_by_name['item'].message_type = schema__pb2._ZENTRY
_ZITEMEXT.fields_by_name['timestamp'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_ZITEMEXTLIST.fields_by_name['items'].message_type = _ZITEMEXT
DESCRIPTOR.message_types_by_name['TamperReport'] = _TAMPERREPORT
DESCRIPTOR.message_types_by_name['ReportOptions'] = _REPORTOPTIONS
DESCRIPTOR.message_types_by_name['DataValue'] = _DATAVALUE
DESCRIPTOR.message_types_by_name['Data'] = _DATA
DESCRIPTOR.message_types_by_name['Response'] = _RESPONSE
DESCRIPTOR.message_types_by_name['VerifiableItemExt'] = _VERIFIABLEITEMEXT
DESCRIPTOR.message_types_by_name['ItemExt'] = _ITEMEXT
DESCRIPTOR.message_types_by_name['ItemExtList'] = _ITEMEXTLIST
DESCRIPTOR.message_types_by_name['ZItemExt'] = _ZITEMEXT
DESCRIPTOR.message_types_by_name['ZItemExtList'] = _ZITEMEXTLIST
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

TamperReport = _reflection.GeneratedProtocolMessageType('TamperReport', (_message.Message,), {
  'DESCRIPTOR' : _TAMPERREPORT,
  '__module__' : 'lc_pb2'
  # @@protoc_insertion_point(class_scope:lc.schema.TamperReport)
  })
_sym_db.RegisterMessage(TamperReport)

ReportOptions = _reflection.GeneratedProtocolMessageType('ReportOptions', (_message.Message,), {
  'DESCRIPTOR' : _REPORTOPTIONS,
  '__module__' : 'lc_pb2'
  # @@protoc_insertion_point(class_scope:lc.schema.ReportOptions)
  })
_sym_db.RegisterMessage(ReportOptions)

DataValue = _reflection.GeneratedProtocolMessageType('DataValue', (_message.Message,), {
  'DESCRIPTOR' : _DATAVALUE,
  '__module__' : 'lc_pb2'
  # @@protoc_insertion_point(class_scope:lc.schema.DataValue)
  })
_sym_db.RegisterMessage(DataValue)

Data = _reflection.GeneratedProtocolMessageType('Data', (_message.Message,), {
  'DESCRIPTOR' : _DATA,
  '__module__' : 'lc_pb2'
  # @@protoc_insertion_point(class_scope:lc.schema.Data)
  })
_sym_db.RegisterMessage(Data)

Response = _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
  'DESCRIPTOR' : _RESPONSE,
  '__module__' : 'lc_pb2'
  # @@protoc_insertion_point(class_scope:lc.schema.Response)
  })
_sym_db.RegisterMessage(Response)

VerifiableItemExt = _reflection.GeneratedProtocolMessageType('VerifiableItemExt', (_message.Message,), {
  'DESCRIPTOR' : _VERIFIABLEITEMEXT,
  '__module__' : 'lc_pb2'
  # @@protoc_insertion_point(class_scope:lc.schema.VerifiableItemExt)
  })
_sym_db.RegisterMessage(VerifiableItemExt)

ItemExt = _reflection.GeneratedProtocolMessageType('ItemExt', (_message.Message,), {
  'DESCRIPTOR' : _ITEMEXT,
  '__module__' : 'lc_pb2'
  # @@protoc_insertion_point(class_scope:lc.schema.ItemExt)
  })
_sym_db.RegisterMessage(ItemExt)

ItemExtList = _reflection.GeneratedProtocolMessageType('ItemExtList', (_message.Message,), {
  'DESCRIPTOR' : _ITEMEXTLIST,
  '__module__' : 'lc_pb2'
  # @@protoc_insertion_point(class_scope:lc.schema.ItemExtList)
  })
_sym_db.RegisterMessage(ItemExtList)

ZItemExt = _reflection.GeneratedProtocolMessageType('ZItemExt', (_message.Message,), {
  'DESCRIPTOR' : _ZITEMEXT,
  '__module__' : 'lc_pb2'
  # @@protoc_insertion_point(class_scope:lc.schema.ZItemExt)
  })
_sym_db.RegisterMessage(ZItemExt)

ZItemExtList = _reflection.GeneratedProtocolMessageType('ZItemExtList', (_message.Message,), {
  'DESCRIPTOR' : _ZITEMEXTLIST,
  '__module__' : 'lc_pb2'
  # @@protoc_insertion_point(class_scope:lc.schema.ZItemExtList)
  })
_sym_db.RegisterMessage(ZItemExtList)


DESCRIPTOR._options = None

_LCSERVICE = _descriptor.ServiceDescriptor(
  name='LcService',
  full_name='lc.schema.LcService',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_start=982,
  serialized_end=2264,
  methods=[
  _descriptor.MethodDescriptor(
    name='Set',
    full_name='lc.schema.LcService.Set',
    index=0,
    containing_service=None,
    input_type=schema__pb2._SETREQUEST,
    output_type=schema__pb2._TXMETADATA,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='Get',
    full_name='lc.schema.LcService.Get',
    index=1,
    containing_service=None,
    input_type=schema__pb2._KEYREQUEST,
    output_type=schema__pb2._ENTRY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='VerifiableSet',
    full_name='lc.schema.LcService.VerifiableSet',
    index=2,
    containing_service=None,
    input_type=schema__pb2._VERIFIABLESETREQUEST,
    output_type=schema__pb2._VERIFIABLETX,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='VerifiableGet',
    full_name='lc.schema.LcService.VerifiableGet',
    index=3,
    containing_service=None,
    input_type=schema__pb2._VERIFIABLEGETREQUEST,
    output_type=schema__pb2._VERIFIABLEENTRY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetAll',
    full_name='lc.schema.LcService.GetAll',
    index=4,
    containing_service=None,
    input_type=schema__pb2._KEYLISTREQUEST,
    output_type=schema__pb2._ENTRIES,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='ExecAll',
    full_name='lc.schema.LcService.ExecAll',
    index=5,
    containing_service=None,
    input_type=schema__pb2._EXECALLREQUEST,
    output_type=schema__pb2._TXMETADATA,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='Scan',
    full_name='lc.schema.LcService.Scan',
    index=6,
    containing_service=None,
    input_type=schema__pb2._SCANREQUEST,
    output_type=schema__pb2._ENTRIES,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='History',
    full_name='lc.schema.LcService.History',
    index=7,
    containing_service=None,
    input_type=schema__pb2._HISTORYREQUEST,
    output_type=schema__pb2._ENTRIES,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='ZAdd',
    full_name='lc.schema.LcService.ZAdd',
    index=8,
    containing_service=None,
    input_type=schema__pb2._ZADDREQUEST,
    output_type=schema__pb2._TXMETADATA,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='VerifiableZAdd',
    full_name='lc.schema.LcService.VerifiableZAdd',
    index=9,
    containing_service=None,
    input_type=schema__pb2._VERIFIABLEZADDREQUEST,
    output_type=schema__pb2._VERIFIABLETX,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='ZScan',
    full_name='lc.schema.LcService.ZScan',
    index=10,
    containing_service=None,
    input_type=schema__pb2._ZSCANREQUEST,
    output_type=schema__pb2._ZENTRIES,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='CurrentState',
    full_name='lc.schema.LcService.CurrentState',
    index=11,
    containing_service=None,
    input_type=google_dot_protobuf_dot_empty__pb2._EMPTY,
    output_type=schema__pb2._IMMUTABLESTATE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='Health',
    full_name='lc.schema.LcService.Health',
    index=12,
    containing_service=None,
    input_type=google_dot_protobuf_dot_empty__pb2._EMPTY,
    output_type=schema__pb2._HEALTHRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='ReportTamper',
    full_name='lc.schema.LcService.ReportTamper',
    index=13,
    containing_service=None,
    input_type=_REPORTOPTIONS,
    output_type=google_dot_protobuf_dot_empty__pb2._EMPTY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='SendData',
    full_name='lc.schema.LcService.SendData',
    index=14,
    containing_service=None,
    input_type=_DATA,
    output_type=_RESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='VerifiableGetExt',
    full_name='lc.schema.LcService.VerifiableGetExt',
    index=15,
    containing_service=None,
    input_type=schema__pb2._VERIFIABLEGETREQUEST,
    output_type=_VERIFIABLEITEMEXT,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='ZScanExt',
    full_name='lc.schema.LcService.ZScanExt',
    index=16,
    containing_service=None,
    input_type=schema__pb2._ZSCANREQUEST,
    output_type=_ZITEMEXTLIST,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='HistoryExt',
    full_name='lc.schema.LcService.HistoryExt',
    index=17,
    containing_service=None,
    input_type=schema__pb2._HISTORYREQUEST,
    output_type=_ITEMEXTLIST,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
])
_sym_db.RegisterServiceDescriptor(_LCSERVICE)

DESCRIPTOR.services_by_name['LcService'] = _LCSERVICE

# @@protoc_insertion_point(module_scope)
